import QtQuick 1.1

import qb.base 1.0
import qb.components 1.0
import BasicUIControls 1.0;

Widget {
	id: thermostatSidePanel

	property string kpiPrefix: "ThermostatSidePanel."
	property ThermostatApp app;
	property alias stateInfoText: thermostatStateInfoText.text
	property alias programOn: toggle.isSwitchedOn

	anchors.fill: parent

	QtObject {
		id: p

		property bool setpointInProgress: false
		property real currentTemperature : 6
		property real currentSetpoint : 6
		property string errMsg: ''
		property string flameIconState: ""

		//for unit tests
		property alias flameIconSource : flameIcon.source

		// when one of the buttons (temperatureValue) is pressed
		function onTemperatureButtonPressed() {
			if (setpointInProgress === false) {
				temperatureValue.value = currentSetpoint;
			}
			setpointInProgress = true;
		}

		// when one of the buttons (temperatureValue) is released
		function onTemperatureButtonReleased() {
			var spChanged = (currentSetpoint != temperatureValue.value)
			currentSetpoint = temperatureValue.value;
			temperatureValue.value = currentTemperature;
			setpointInProgress = false;
			if (spChanged)
				app.sendSetPoint(currentSetpoint);
		}

		function activeTemperaturePreset(id) {
			tempTileAway.state = (id === tempTileAway.stateId) ? "down" : "up";
			tempTileSleep.state = (id === tempTileSleep.stateId) ? "down" : "up";
			tempTileHome.state = (id === tempTileHome.stateId) ? "down" : "up";
			tempTileComfort.state = (id === tempTileComfort.stateId) ? "down" : "up";
		}

		function onTemperaturePresetChanged(id) {
			activeTemperaturePreset(id);
			app.sendTempState(id);
		}

		function onDatasetThermInfoChanged() {
//Joop			currentTemperature = app.thermInfo['currentDisplayTemp'] / 100.0;
			currentTemperature = app.thermInfo['currentTemp'] / 100.0;
			currentSetpoint = app.thermInfo['currentSetpoint'] / 100.0;

			activeTemperaturePreset(app.thermInfo['activeState']);

			var flameState = "";

			var modulationLevel = app.thermInfo['currentModulationLevel'];
			// burnerInfo 0=off, 1=heat, 2=water, 3=preheat, 4=error
			var burnerInfo = app.thermInfo['burnerInfo'];
			//When burner is set to on (heating or preheating) use the modulation level as a guide
			switch (burnerInfo) {
			case 1:
			case 3:
				//Show flames based on modulation level
				if (modulationLevel > 70) {
					flameState = "flameLarge";
				} else if (modulationLevel > 35) {
					flameState = "flameMedium";
				} else {
					flameState = "flameSmall";
				}
				break;
			case 2:
				//Heating water
				flameState = "flameDhw";
				break;
			case 4:
				//Error
				flameState = "flameError";
				break;
			}

			if (app.thermInfo.boilerModuleConnected == 0) {
				setPoint.text = qsTr("Error");
				setPoint.state = 'error';
				errMsg = qsTr("Boiler adapter not connected.");
				rectStateInfo.state = 'error';
				flameState = "flameError";
			} else if (app.thermInfo.haveOTBoiler == "1" && app.thermInfo.otCommError == "1") {
				if (app.boilerManagementFeature) {
					setPoint.text = '';
					setPoint.state = 'invisible';
					errMsg = qsTr("There is a problem communicating with your boiler");
				} else {
					setPoint.text = qsTr("Error");
					setPoint.state = 'error';
					errMsg = qsTr("Boiler not connected.");
				}
				rectStateInfo.state = 'error';
				flameState = "flameError";
			} else if (app.thermInfo.haveOTBoiler == "1" && app.thermInfo.errorFound != 255 && !app.boilerManagementFeature) {
				setPoint.text = qsTr("Errorcode: %1").arg(app.thermInfo.errorFound);
				setPoint.state = 'error';
				errMsg = qsTr("Boiler error see manual.");
				rectStateInfo.state = 'error';
				flameState = "flameError";
			} else if (app.boilerManagementFeature && app.boilerErrorInfo.errorActive) {
				setPoint.text = '';
				setPoint.state = 'invisible';
				if (app.boilerErrorInfo.SCDataReady)
					errMsg = app.boilerErrorInfo.errorTitle;
				else
					errMsg = qsTr("SC_timeout_title");
				rectStateInfo.state = 'error';
				flameState = "flameError";
			} else {
				setThermostatState();
			}

			p.flameIconState = flameState;
		}

		function showSetPointText()	{
			setPoint.text = qsTr("> %1°").arg(i18n.number(currentSetpoint, 1));
			setPoint.state = 'setpoint';
		}

		function isStateSameAsCurrentOrNextProgramState(state) {
			if (state === app.thermStateUndef)
				return false;
			var now = new Date();
			var nowDayOfWeek = now.getDay();
			var nowTime = (now.getHours() * 60) + now.getMinutes();
			var dayOfWeekIdx = nowDayOfWeek;
			for (var i = 0; i < app.thermostatProgram[dayOfWeekIdx].length; i++) {
				// calculate the day offset between this program's start day and dayOfWeekIdx
				// negative days means it starts before dayOfWeekIdx
				var startDaysOffset = app.thermostatProgram[dayOfWeekIdx][i].startDayOfWeek - dayOfWeekIdx;
				// if we get a positive value, it means the start days is in the previous week, so we
				// adjust
				if (startDaysOffset > 0)
					startDaysOffset -= 7;
				// calculate the relative start time based on dayOfWeekIdx's midnight
				var startTime = 0;
				if (startDaysOffset < 0)
					startTime -= 1440;
				startTime += (app.thermostatProgram[dayOfWeekIdx][i].startHour * 60) + app.thermostatProgram[dayOfWeekIdx][i].startMin;
				startTime += (startDaysOffset < 0 ? startDaysOffset + 1 : startDaysOffset) * 1440;

				// calculate how many days ahead of dayOfWeekIdxthis program ends
				var endDaysOffset = app.thermostatProgram[dayOfWeekIdx][i].endDayOfWeek - dayOfWeekIdx;
				// if we get a negative value, it means the end day is in the next week, so we
				// "add" 7 days
				if (endDaysOffset < 0)
					endDaysOffset += 7;
				// calculate the relative end time based on dayOfWeekIdx's midnight
				var endTime = 0;
				if (endDaysOffset < 0)
					endTime -= 1440;
				endTime += (app.thermostatProgram[dayOfWeekIdx][i].endHour * 60) + app.thermostatProgram[dayOfWeekIdx][i].endMin;
				endTime += endDaysOffset * 1440;

				// add an offset to adjust the times according to today's midnight
				var nowOffset = dayOfWeekIdx - nowDayOfWeek;
				if (nowOffset < 0)
					nowOffset += 7
				startTime += nowOffset * 1440;
				endTime += nowOffset * 1440;

				if (startTime <= nowTime && endTime > nowTime) {
					// here we are looking at the current program

					// when states match, we know its an override of the current program with the same state
					if (app.thermostatProgram[dayOfWeekIdx][i].targetState === state) {
						return true;
					// if we are at a program that only ends at a later day, restart loop with that day and second program
					} else if (endTime >= 1440) {
						dayOfWeekIdx = (dayOfWeekIdx + Math.floor(endTime / 1440)) % 7;
						i = 0;
					}
				} else if (startTime > nowTime) {
					// here we are at the next program (after current one)

					// when the next state is the same as the current state, we got blended
					return (app.thermostatProgram[dayOfWeekIdx][i].targetState === state);
				}
			}

			return false;
		}

		function setThermostatState() {
			var thermostatStateString = '';

			setPoint.state = 'invisible';
			rectStateInfo.state = 'normal';

			switch(app.thermInfo['programState']) {

			case app.progStateBaseScheme:
				// when tapping hot water, keep the same text as it was before
				if (app.thermInfo['burnerInfo'] === app.burnerDhw && thermostatStateInfoText.text.length > 0) {
					thermostatStateString = thermostatStateInfoText.text;
					break;
				}
				if (app.thermInfo['burnerInfo'] === app.burnerPreheat) {
					thermostatStateString = nextBlockPreheating();
					showSetPointText();
				} else {
					if (app.thermInfo['currentSetpoint'] < app.thermInfo['currentDisplayTemp'])
						showSetPointText();
					thermostatStateString = nextBlock();
				}
				break;

			case app.progStateTemperatureOverride:
				if (app.thermInfo['burnerInfo'] === app.burnerPreheat) {
					thermostatStateString = nextBlockPreheating();
					showSetPointText();
				} else if (app.thermInfo['currentSetpoint'] > app.thermInfo['currentDisplayTemp'] &&
						isStateSameAsCurrentOrNextProgramState(app.thermInfo['activeState']) === false) {
					thermostatStateString = qsTr("Temporarily to %1°").arg(i18n.number(currentSetpoint, 1));
					showSetPointText();
				} else {
					if (app.thermInfo['currentSetpoint'] !== app.thermInfo['currentDisplayTemp'])
						showSetPointText();
					thermostatStateString = nextBlock();
				}
				break;

			case app.progStateManualControl:
			case app.progStateLockedBaseScheme:
				if (app.thermInfo['currentSetpoint'] > app.thermInfo['currentDisplayTemp']) {
					thermostatStateString = qsTr("To %1°").arg(i18n.number(currentSetpoint, 1));
				} else {
					thermostatStateString = qsTr("On %1°").arg(i18n.number(currentSetpoint, 1));
				}
				break;

			case app.progStateHoliday:
				thermostatStateString = qsTr("Vacation until %1").arg(vacationEnd());
				break;

			default:
				break;
			}
			thermostatStateInfoText.text = thermostatStateString;
		}

		function nextBlockPreheating() {
			var nextProgramStart = new Date(app.thermInfo['nextTime'] * 1000);
			var now = new Date();
			var diff = nextProgramStart - now; // milliseconds
			diff = Math.floor((diff/1000)/60); // minutes
			diff = Math.max(Math.round(diff / 5) * 5, 5); // 5 minutes block
			if (diff >= 60) {
				var diffHr = Math.floor(diff / 60);
				var diffMin = diff % 60;
				if(diffMin !== 0)
					return qsTr("To %1° in %2h %3min.").arg(i18n.number(app.thermInfo['nextSetpoint'] / 100, 1))
					.arg(diffHr.toString())
					.arg(diffMin.toString());
				else
					return qsTr("To %1° in %2h").arg(i18n.number(app.thermInfo['nextSetpoint'] / 100, 1))
					.arg(diffHr.toString());
			} else {
				return qsTr("To %1° in %2 min.").arg(i18n.number(app.thermInfo['nextSetpoint'] / 100, 1)).arg(diff.toString());
			}
		}

		function nextBlock() {
			return qsTr("At %1 set to %2°").arg(dateOrTime(app.thermInfo['nextTime'] * 1000))
			.arg(i18n.number(app.thermInfo['nextSetpoint'] / 100, 1));
		}

		function isDiffMoreThan24Hrs(compareTime) {
			var nowPlus24hrs = new Date();
			nowPlus24hrs.setDate(nowPlus24hrs.getDate() + 1);
			return compareTime > nowPlus24hrs.getTime();
		}

		function dateOrTime(time) {
			var dateOrTimeStr = ""
			var jsTime = new Date(time);

			if (isDiffMoreThan24Hrs(time)) {
				dateOrTimeStr = i18n.dateTime(jsTime, i18n.time_no | i18n.date_yes | i18n.year_no | i18n.mon_short);
			} else {
				dateOrTimeStr = i18n.dateTime(jsTime, i18n.time_yes | i18n.secs_no | i18n.hour_str_yes);
			}
			return dateOrTimeStr;
		}

		function vacationEnd() {
			var nowPlus3Years = new Date();
			nowPlus3Years.setFullYear(nowPlus3Years.getFullYear() + 3);
			var vacationEnd = new Date(app.thermInfo['nextTime'] * 1000)
			if (nowPlus3Years < vacationEnd) {
				return qsTr("I return");
			} else {
				return dateOrTime(app.thermInfo['nextTime'] * 1000);
			}
		}

		onCurrentTemperatureChanged: {
			if (setpointInProgress === false) {
				temperatureValue.value = currentTemperature;
			}
		}
	}

	function init() {
		app.thermInfoChanged.connect(p.onDatasetThermInfoChanged);
		app.boilerErrorInfoChanged.connect(p.onDatasetThermInfoChanged);
		app.boilerManagementFeatureChanged.connect(p.onDatasetThermInfoChanged);
		temperatureValue.buttonPressed.connect(p.onTemperatureButtonPressed);
		temperatureValue.buttonReleased.connect(p.onTemperatureButtonReleased);
	}

	Item {
		id: normalSidePanel
		width: parent.width
		height: parent.height
		visible: !dimState
		anchors.top: parent.top
		anchors.topMargin: 15

		NumberSpinner {
			id: temperatureValue
			width: 248
			height: 114

			spacing: 4
			radius: 6
			buttonWidth: 80


			bottomLeftRadiusRatio: 0
			bottomRightRadiusRatio: 0

			fontFamily: qfont.regular.name
			fontPixelSize: 50
			fontColor: colors.spText
			textBaseline: 83

			backgroundColor:            colors.spBackgroundValue
			backgroundColorButtonsUp:   colors.spBackgroundButtonsUp
			backgroundColorButtonsDown: colors.spBackgroundButtonsDown
			overlayColorButtonsUp:      colors.spOverlayButtonsUp
			overlayColorButtonsDown:    colors.spOverlayButtonsDown

			disableButtonAtMaximum: true
			disableButtonAtMinimum: true

			downButtonBottomClickMargin: 2

			borderColorUp: colors.spSpinnerBorderUp
			borderColorDown: colors.spSpinnerBorderDown
			borderWidth: 1
			borderStyle: Qt.SolidLine

			pressingEndTime: 3000

			rangeMin: 6.0
			rangeMax: 30.0
			increment: 0.5
			valueSuffix: "°"

			mouseIsActiveInDimState: true

			upIconSource: "drawables/Plus.png"
			downIconSource: "drawables/Minus.png"

			Image {
				id: flameIcon
				anchors.top: parent.top
				anchors.topMargin: 10
				anchors.left: parent.left
				anchors.leftMargin: 133
				source: p.flameIconState.length > 0 ? "drawables/" + p.flameIconState + ".png" : "";
			}

			Text {
				id: setPoint
				visible: false
				anchors {
					bottom: parent.valueField.bottom
					bottomMargin: 5
					right: parent.valueField.right
					rightMargin: 5
					left: parent.valueField.left
					leftMargin: 5
				}
				font.family: qfont.regular.name
				font.pixelSize: 15
				color: colors.spText
				text: "> 19,0°"
				horizontalAlignment:  Text.AlignRight
				state: 'invisible'
				states: [
					State {
						name: 'invisible'
						PropertyChanges {
							target: setPoint
							visible: false
						}
					},
					State {
						name: 'setpoint'
						PropertyChanges {
							target: setPoint
							visible: true
						}
					},
					State {
						name: 'error'
						PropertyChanges {
							target: setPoint
							visible: true
						}
						PropertyChanges {
							target: setPoint
							horizontalAlignment: Text.AlignHCenter
						}
						PropertyChanges {
							target: setPoint
							color: colors.spErrorText
						}
					}
				]
			}
		}


		StyledRectangle {
			id: rectStateInfo
			width: 248
			height: 74

			anchors.top: temperatureValue.bottom
			anchors.topMargin: 4

			color: colors.spBackgroundValue
			radius: 6
			topLeftRadiusRatio: 0
			topRightRadiusRatio: 0

			Item {
				id: normalState
				anchors.fill: parent
				visible: true
				MouseArea {
					property string kpiPostfix: "programLine." + rectStateInfo.state
					anchors.fill: parent
					onClicked: {
						if (app && app.programScreen) {
							app.programScreen.show();
						}

					}
				}

				Text {
					anchors.left: parent.left
					anchors.leftMargin: 12
					anchors.baseline: parent.top
					anchors.baselineOffset: 26

					font.family: qfont.semiBold.name
					font.pixelSize: 17
					color: colors.spText

					text: qsTr("Program")
				}

				Text {
					id: thermostatStateInfoText

					anchors.left: parent.left
					anchors.leftMargin: 12
					anchors.baseline: parent.bottom
					anchors.baselineOffset: -16

					font.family: qfont.regular.name
					font.pixelSize: 15
					color: colors.spText

					text: "Continue on 19,0"
				}

				OnOffToggle {
					id: toggle

					anchors.top: parent.top
					anchors.topMargin: 7
					anchors.right: parent.right
					anchors.rightMargin: 17

					fontFamily: qfont.semiBold.name
					fontPixelSize: 17
					fontColor: colors.spText

					leftTextOff: qsTr("Off")
					leftTextOn: qsTr("On")
					topSpacing: 8

					isSwitchedOn: app.programEnabled
					positionIsLeft: true
					leftIsSwitchedOn: false
					mouseIsActiveInDimState: true

					onSelectedChangedByUser: app.sendProgramState(isSwitchedOn)
				}
			}
			Item {
				id: errorState
				anchors.fill: parent
				visible: false
				MouseArea {
					id: mouseAreaBM
					enabled: app.boilerManagementFeature
					property string kpiPostfix: "programLine." + rectStateInfo.state
					anchors.fill: parent
					onClicked: {
						stage.openFullscreen(Qt.resolvedUrl("../boiler/BoilerScreen.qml"), {showDefault: true});
					}
				}
				Text {
					id: errorMsg
					anchors.fill: parent
					anchors.leftMargin: 10
					anchors.rightMargin: 10
					verticalAlignment: Text.AlignVCenter
					horizontalAlignment: Text.AlignHCenter
					wrapMode: Text.WordWrap
					elide: Text.ElideRight
					maximumLineCount: 3
					text: p.errMsg
					font.family: qfont.regular.name
					font.pixelSize: 15
					color: colors.spText
				}
			}
			state: 'normal'
			states: [
				State {
					name: 'normal'
					PropertyChanges {
						target: normalState
						visible: true
					}
					PropertyChanges {
						target: errorState
						visible: false
					}
				},
				State {
					name: 'error'
					PropertyChanges {
						target: normalState
						visible: false
					}
					PropertyChanges {
						target: errorState
						visible: true
					}
				}
			]
		}

		Item {
			anchors {
				top: rectStateInfo.bottom
				topMargin: 16
				left: rectStateInfo.left
				right: rectStateInfo.right
			}
			height: 176

			TemperatureRectangle {
				id: tempTileAway
				anchors.top: parent.top
				anchors.left: parent.left

				topLeftRadiusRatio: 1

				subLabelText: app.thermStateName[app.thermStateAway]
				temperature: app.thermStates.thermStateAway.temperature
				stateId: app.thermStates.thermStateAway.index

				onPressed: p.onTemperaturePresetChanged(tempTileAway.stateId)
			}

			TemperatureRectangle {
				id: tempTileHome
				anchors.top: parent.top
				anchors.left: tempTileAway.right
				anchors.leftMargin: 4

				topRightRadiusRatio: 1

				subLabelText: app.thermStateName[app.thermStateActive]
				temperature: app.thermStates.thermStateActive.temperature
				stateId: app.thermStates.thermStateActive.index

				onPressed: p.onTemperaturePresetChanged(tempTileHome.stateId)
			}

			TemperatureRectangle {
				id: tempTileSleep
				anchors.top: tempTileAway.bottom
				anchors.topMargin: 4
				anchors.left: parent.left

				bottomLeftRadiusRatio: 1

				subLabelText: app.thermStateName[app.thermStateSleep]
				temperature: app.thermStates.thermStateSleep.temperature
				stateId: app.thermStates.thermStateSleep.index

				onPressed: p.onTemperaturePresetChanged(tempTileSleep.stateId)
			}

			TemperatureRectangle {
				id: tempTileComfort
				anchors.top: tempTileHome.bottom
				anchors.topMargin: 4
				anchors.left: tempTileSleep.right
				anchors.leftMargin: 4

				bottomRightRadiusRatio: 1

				subLabelText: app.thermStateName[app.thermStateRelax]
				temperature: app.thermStates.thermStateRelax.temperature
				stateId: app.thermStates.thermStateRelax.index

				onPressed: p.onTemperaturePresetChanged(tempTileComfort.stateId)
			}
			Item {
				id: blockingBanner
				anchors {
					fill: parent
					// -1 to cover temp preset shadow if any
					rightMargin: -1
					bottomMargin: -1
				}

				visible: !app.thermostatStatesSaved
				MouseArea {
					id: maPresetBlocker
					anchors.fill: parent
				}
				Rectangle {
					id: blockingBannerBckg
					anchors.fill: parent
					radius: 6
					//TODO: real color
					color: "white"
					opacity: 0.6
				}
				StandardButton {
					text: qsTr("Set")
					anchors.centerIn: parent
					onClicked: {
						stage.openFullscreen(app.temperaturePresetScreenUrl);
					}
				}
			}
		}
	}

	Item {
		id: dimSidePanel
		visible: !normalSidePanel.visible
		height: parent.height
		width: parent.width

		Text {
			id: temperatureValueDim
			anchors {
				baseline: parent.top
				//from the top of the screen (screen topBar included)
//waste display start
//				baselineOffset: 233 - 56
				baselineOffset: app.wasteIconShow ? 233 - 106 : 233 - 56
//waste display end
				left: parent.right
				leftMargin: -252
			}
			font.family: qfont.regular.name
			font.pixelSize: 75
			color: colors.spText
			text: temperatureValue.valueField.text
		}

		Image {
			id: flameIconDim
			anchors {
				bottom: temperatureValueDim.top
				bottomMargin: -25
				left: temperatureValueDim.right
				leftMargin: -8
			}
			source: p.flameIconState.length > 0 ? "drawables/" + p.flameIconState + "_dim.png" : "";
		}

		Text {
			id: thermostatStateInfoTextDim
			anchors {
				left: temperatureValueDim.left
				baseline: parent.bottom
				//from the bottom of the screen (screen bottomBar included)
//waste display start
//				baselineOffset: -209 + 10
				baselineOffset: app.wasteIconShow ? -209 - 40 : -209 + 10
//waste display end
			}
			font.family: qfont.regular.name
			font.pixelSize: 20
			width: 220
			horizontalAlignment: Text.AlignLeft
			wrapMode: Text.WordWrap
			color: colors.spText
			visible: rectStateInfo.state == "normal"
			text: thermostatStateInfoText.text
		}

		Text {
			id: errorStateTextDim
			anchors {
				left: temperatureValueDim.left
				top: errorCodeTextDim.bottom
				topMargin: 5
			}
			font.family: qfont.regular.name
			font.pixelSize: 18
			width: 220
			horizontalAlignment: Text.AlignLeft
			wrapMode: Text.WordWrap
			elide: Text.ElideRight
			maximumLineCount: 3
			lineHeightMode: Text.FixedHeight
			lineHeight: 24
			color: colors.spText
			visible: !thermostatStateInfoTextDim.visible
			text: errorMsg.text
		}

		Text {
			id: errorCodeTextDim
			anchors {
				left: temperatureValueDim.left
				right: temperatureValueDim.right
				top: temperatureValueDim.bottom
			}
			font.family: qfont.semiBold.name
			font.pixelSize: 18
			horizontalAlignment: Text.Center
			color: colors.spText
			visible: setPoint.state == 'error'
			text: setPoint.text
		}
//waste display start

 		Image {
       	 	id: wasteMessageIcon
			height: 80
			width: 80
			anchors {
				left: temperatureValueDim.left
				right: temperatureValueDim.right
				top: errorCodeTextDim.bottom
				topMargin: 20
			}
			fillMode: Image.PreserveAspectFit
			source: app.wasteIcon
		}
//waste display end

	}
}
